/*
 * USAGE.C
 *
 * Colorful, sorted and optionally rich directory enumeration
 * for Windows.
 *
 * This module contains help text to display to the user for invalid arguments
 * or upon request.
 *
 * Copyright (c) 2014-16 Malcolm J. Smith
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include "sdir.h"

#ifndef SDIR_NO_HELP

#define SDIR_USAGE_HEADER_COLOR (BACKGROUND_BLUE|FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_INTENSITY)
SDIR_COLOR_ATTRIBUTES SdirUsageHeaderColor = {0, SDIR_USAGE_HEADER_COLOR};

BOOL
SdirUsageVersionInfo()
{
    TCHAR   Line[160];

    _stprintf_s(Line,
                sizeof(Line)/sizeof(Line[0]),
                _T("Sdir version %i.%i, compiled %hs\n")
                _T("  Feature level %i, %hs, %hs"),
                SDIR_VER_MAJOR,
                SDIR_VER_MINOR,
                __DATE__,
                SDIR_FEATURE_LEVEL,
#ifdef UNICODE
                "Unicode",
#else
                "ANSI",
#endif
#ifdef MINICRT
                "Minicrt"
#else
                "Msvcrt"
#endif
                );

    if (!SdirWriteString(Line)) {
        return FALSE;
    }

#ifdef _MSC_VER
    _stprintf_s(Line,
                sizeof(Line)/sizeof(Line[0]),
                _T(", C version %i.%02i"),
                _MSC_VER / 100,
                _MSC_VER % 100);

    if (!SdirWriteString(Line)) {
        return FALSE;
    }
#endif

    if (!SdirWriteString(_T("\n"))) {
        return FALSE;
    }

    return TRUE;
}

BOOL
SdirUsageHeader(
    LPCSTR strHeader
    )
{
    TCHAR   Line[80];
    DWORD   Padding;
    DWORD   MaxWidth = 80;
    BOOLEAN ExtraNewline = FALSE;

    //
    //  Write a newline with default color.
    //

    if (!SdirWriteString(_T("\n"))) {
        return FALSE;
    }

    //
    //  Now write the heading with desired color.
    //

    _stprintf_s(Line, sizeof(Line)/sizeof(Line[0]), _T("  === %hs ==="), strHeader);
    if (!SdirWriteStringWithAttribute(Line, SdirUsageHeaderColor)) {
        return FALSE;
    }

    Padding = (DWORD)_tcslen(Line) - 1;
    if (Opts->ConsoleBufferWidth > Padding) {

        //
        //  If the console will not auto wrap for us, we need an extra newline.
        //  If it will, just skip that part.
        //

        if (Opts->ConsoleBufferWidth > MaxWidth) {
            Padding = MaxWidth - Padding;
            ExtraNewline = TRUE;
        } else {
            Padding = Opts->ConsoleBufferWidth - Padding;
        }

        while (Padding > 0) {
            if (!SdirWriteStringWithAttribute(_T(" "), SdirUsageHeaderColor)) {
                return FALSE;
            }
            Padding--;
        }
    }

    //
    //  If we're outputting to a file, assume a newline is always needed
    //  since we won't have wrapping behavior.
    //
    
    if (!Opts->OutputHasAutoLineWrap) {
        ExtraNewline = TRUE;
    }

    if (!SdirWriteString(_T("\n"))) {
        return FALSE;
    }

    if (ExtraNewline && !SdirWriteString(_T("\n"))) {
        return FALSE;
    }

    return TRUE;
}


const
CHAR strHelpHeader[] = "HELP OPTIONS";

const
CHAR strHelpUsage1[] = 
                   "  Usage: [opts] [pathspec] ...\n"
                   "\n"
                   "  For additional help:\n"
                   "\n"
                   "   -opts        General command line options\n"
                   "   -display     Options to display metadata attributes\n"
#ifdef SDIR_SUPPORT_CUSTOM_FILECOLOR
                   "   -filecolor   Configuration information for color of each file\n"
#endif
#ifdef SDIR_SUPPORT_CUSTOM_METACOLOR
                   "   -metacolor   Configuration information for metadata attributes\n"
#endif
                   "   -license     The license to use the software\n"
#ifdef SDIR_SUPPORT_UPDATE
                   "   -online      Options to download or update from the internet\n"
#endif
                   "   -sort        Options to sort files\n"
#ifdef SDIR_SUPPORT_VT
                   "   -vt          Configuration when communicating with VT100 terminals\n"
#endif
                   "\n"
                   "   -all         Display all help topics\n";

BOOL
SdirUsageHelp()
{
    TCHAR   strUsage[1024];

    if (!SdirUsageHeader(strHelpHeader)) {
        return FALSE;
    }

    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strHelpUsage1);

    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    return TRUE;
}


const
CHAR strCmdLineHeader[] = "COMMAND LINE OPTIONS";

const
CHAR strCmdLineOpts[] = 
                   "  Usage: [opts] [pathspec] ...\n"
                   "\n"
                   "   -?           Display help\n"
                   "\n"
#ifdef SDIR_SUPPORT_RECURSIVE
                   "   -br[num]     Brief recurse, limit depth to num\n"
                   "   -bs[size]    Brief recurse, limit size to size\n"
#endif
                   "   -cw[num]     Width of console when writing to files\n"
#ifdef SDIR_SUPPORT_CUSTOM_FILECOLOR
                   "   -fc[string]  Apply custom file color string, see file color section\n"
                   "   -fe[string]  Exclude files matching criteria, see file color section\n"
#endif
                   "   -l/-ln       Traverse symbolic links and mount points when recursing\n"
                   "   -p/-pn       Pause/no pause after each screen\n"
#ifdef SDIR_SUPPORT_RECURSIVE
                   "   -r[dir]      Recurse through dir when enumerating\n"
#endif
                   "   -t/-tn       Truncate/no truncate of very long file names\n"
#ifdef UNICODE
                   "   -u/-un       Unicode/no unicode output\n"
#endif
                   "   -v           Display version/build info and exit\n"
                   "";

const
CHAR strCmdLineUsage2[] = "\n"
                   " Options can also be in the SDIR_OPTS environment variable.\n"
                   " Processed in order, environment then arguments.\n";


BOOL
SdirUsageOpts()
{
    TCHAR   strUsage[1024];

    if (!SdirUsageHeader(strCmdLineHeader)) {
        return FALSE;
    }

    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strCmdLineOpts);

    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    return TRUE;
}

#ifdef SDIR_SUPPORT_UPDATE
const
CHAR strOnlineHeader[] = "ONLINE OPTIONS";

const
CHAR strOnlineUsage[] =
                   "   -get-update  Update to latest version of current type\n"
                   "   -get-daily   Get the latest daily version\n"
                   "   -get-source  Get source for this version\n"
                   "   -get-stable  Get the latest stable version\n"
                   "   -get-symbol  Get debugging symbol for this version\n"
                   "   -get-ver-x.y Get a specific version of the program\n";

BOOL
SdirUsageOnline()
{
    TCHAR strUsage[1024];

    if (!SdirUsageHeader(strOnlineHeader)) {
        return FALSE;
    }

    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strOnlineUsage);

    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    return TRUE;
}
#endif

const
CHAR strLicenseHeader[] = "LICENSE";

const
CHAR strLicenseText[] = 
     "Copyright (c) 2014-16 Malcolm J. Smith\n"
     "\n"
     "Permission is hereby granted, free of charge, to any person obtaining a copy\n"
     "of this software and associated documentation files (the \"Software\"), to deal\n"
     "in the Software without restriction, including without limitation the rights\n"
     "to use, copy, modify, merge, publish, distribute, sublicense, and/or sell\n"
     "copies of the Software, and to permit persons to whom the Software is\n"
     "furnished to do so, subject to the following conditions:\n"
     "\n"
     "The above copyright notice and this permission notice shall be included in\n"
     "all copies or substantial portions of the Software.\n"
     "\n"
     "THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n"
     "IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n"
     "FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n"
     "AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n"
     "LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\n"
     "OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\n"
     "THE SOFTWARE.\n";

BOOL
SdirUsageLicense()
{
    TCHAR strLicense[2048];

    if (!SdirUsageHeader(strLicenseHeader)) {
        return FALSE;
    }

    SdirAnsiToUnicode(strLicense, sizeof(strLicense)/sizeof(strLicense[0]), strLicenseText);

    if (!SdirWriteString(strLicense)) {
        return FALSE;
    }

    return TRUE;
}

const
CHAR strDisplayHeader[] = "DISPLAY OPTIONS";

BOOL
SdirUsageDisplay()
{
    DWORD   i;
    TCHAR   Line[80];

    if (!SdirUsageHeader(strDisplayHeader)) {
        return FALSE;
    }

    for (i = 0; i < SdirGetNumSdirOptions(); i++) {
        if (SdirOptions[i].Default.Flags & SDIR_FEATURE_ALLOW_DISPLAY) {

            _stprintf_s(Line,
                        sizeof(Line)/sizeof(Line[0]),
                        _T("   -d%s/-h%s    Display/hide %hs%hs\n"),
                        SdirOptions[i].Switch,
                        SdirOptions[i].Switch,
                        SdirOptions[i].Help,
                        (SdirOptions[i].Default.Flags&SDIR_FEATURE_DISPLAY)?" (displayed by default)":"");

            if (!SdirWriteString(Line)) {
                return FALSE;
            }
        }
    }

    return TRUE;
}

const
CHAR strSortHeader[] = "SORT OPTIONS";

BOOL
SdirUsageSort()
{
    DWORD   i;
    TCHAR   strUsage[256];
    TCHAR   Line[80];

    if (!SdirUsageHeader(strSortHeader)) {
        return FALSE;
    }

    _stprintf_s(Line,
                sizeof(Line)/sizeof(Line[0]),
                _T(" Up to %i sort criteria can be applied.  Supported sort flags are:\n"),
                (ULONG)(sizeof(Opts->Sort)/sizeof(Opts->Sort[0])));

    if (!SdirWriteString(Line)) {
        return FALSE;
    }

    for (i = 0; i < SdirGetNumSdirOptions(); i++) {
        if (SdirOptions[i].Default.Flags & SDIR_FEATURE_ALLOW_SORT) {

            _stprintf_s(Line,
                        sizeof(Line)/sizeof(Line[0]),
                        _T("   -s%s/-i%s    Sort/inverse sort by %hs\n"),
                        SdirOptions[i].Switch,
                        SdirOptions[i].Switch,
                        SdirOptions[i].Help);

            if (!SdirWriteString(Line)) {
                return FALSE;
            }
        }
    }

    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strCmdLineUsage2);

    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    return TRUE;
}

#ifdef SDIR_SUPPORT_CUSTOM_FILECOLOR

const
CHAR strFileColorHeader[] = "FILE COLOR CODING";

const
CHAR strFileColorUsage1[] = 
                   " Color coding for files is defined via three environment variables,\n"
                   " plus the command line, processed in order:\n"
                   "   SDIR_COLOR_PREPEND, processed first\n"
                   "   Command line -fc options, processed next\n"
                   "   SDIR_COLOR_REPLACE, if defined; otherwise, built in defaults apply\n"
                   "   SDIR_COLOR_APPEND, processed last\n"
                   "\n"
                   " Each variable contains a semicolon delimited list of rules.  Each rule takes\n"
                   " the form of:\n"
                   "\n"
                   "   [file attribute][operator][criteria],[color]\n"
                   "\n"
                   " The -fe command line option is a shorthand form, which is equivalent to\n"
                   " specifying a color of 'hide', and is condensed to:\n"
                   "\n"
                   "   [file attribute][operator][criteria]\n"
                   "\n"
                   " Valid operators are:\n"
                   "   =   File attribute matches criteria\n"
                   "   !=  File attribute does not match criteria\n"
                   "   >   File attribute greater than criteria\n"
                   "   >=  File attribute greater than or equal to criteria\n"
                   "   <   File attribute less than criteria\n"
                   "   <=  File attribute less than or equal to criteria\n"
                   "   &   File attribute includes criteria or wildcard string\n"
                   "   !&  File attribute does not include criteria or wildcard string\n"
                   "\n"
                   " Valid attributes and corresponding operators are:\n";

const
CHAR strFileColorUsage2[] = "\n"
                   " Multiple colors and keywords can be combined, delimited by +. Valid colors\n"
                   " are:\n";

const
CHAR strFileColorUsage3[] = "\n"
                   " In addition to regular colors, special keywords can be included.\n"
                   " For file colors, these are:\n"
                   "\n"
                   "   bright    To display the file with a bright form of the color\n"
                   "   hide      To suppress a file from display\n"
                   "   invert    To swap background and foreground color\n"
                   "\n"
                   "   continue  To continue matching further rules and merge with subsequent\n"
                   "             results.\n"
                   "\n"
                   " The default set of file color attributes in this build is:\n";


BOOL
SdirUsageFileColor()
{
    TCHAR   Line[80];
    TCHAR   strUsage[2048];
    DWORD   i;
    LPCSTR  This;

    if (!SdirUsageHeader(strFileColorHeader)) {
        return FALSE;
    }

    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strFileColorUsage1);

    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    //
    //  Display supported options and operators
    //

    for (i = 0; i < SdirGetNumSdirOptions(); i++) {
        if (SdirOptions[i].GenerateFromStringFn != NULL) {

            _stprintf_s(Line,
                        sizeof(Line)/sizeof(Line[0]),
                        _T("   %s (%hs), %hs%hs%hs\n"),
                        SdirOptions[i].Switch,
                        SdirOptions[i].Help,
                        SdirOptions[i].CompareFn?"=, !=, >, >=, <, <=":"",
                        (SdirOptions[i].CompareFn && SdirOptions[i].BitwiseCompareFn)?", ":"",
                        SdirOptions[i].BitwiseCompareFn?"&, !&":"");

            if (!SdirWriteString(Line)) {
                return FALSE;
            }
        }
    }
    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strFileColorUsage2);
    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    //
    //  Display supported colors
    //

    for (i = 0; i < 8; i++) {

        SDIR_COLOR_ATTRIBUTES Attr;

        _stprintf_s(Line,
                    sizeof(Line)/sizeof(Line[0]),
                    _T("   %-16s"),
                    ColorString[i].String);

        if (i==0) {
            SdirSetColorToWin32(&Attr, SDIR_DEFAULT_COLOR);
        } else {
            SdirSetColorToWin32(&Attr, (UCHAR)i);
        }

        SdirWriteStringWithAttribute(Line, Attr);

        _stprintf_s(Line,
                    sizeof(Line)/sizeof(Line[0]),
                    _T(" %-16s"),
                    ColorString[i+8].String);

        SdirSetColorToWin32(&Attr, (UCHAR)(i + 8));

        SdirWriteStringWithAttribute(Line, Attr);

        _stprintf_s(Line,
                    sizeof(Line)/sizeof(Line[0]),
                    _T(" bg_%-16s"),
                    ColorString[i].String);

        SdirSetColorToWin32(&Attr, (UCHAR)((i<<4) + 0xF));

        SdirWriteStringWithAttribute(Line, Attr);

        _stprintf_s(Line,
                    sizeof(Line)/sizeof(Line[0]),
                    _T(" bg_%-16s"),
                    ColorString[i+8].String);

        SdirSetColorToWin32(&Attr, (UCHAR)((i+8)<<4));

        SdirWriteStringWithAttribute(Line, Attr);
        if (!SdirWriteString(_T("\n"))) {
            return FALSE;
        }
    }


    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strFileColorUsage3);
    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    //
    //  We want to display built in rules, but for formatting's sake display
    //  one per line.  This is the kludge to make that happen.
    //

    This = SdirAttributeDefaultApplyString;

    _stprintf_s(Line,
                sizeof(Line)/sizeof(Line[0]),
                _T("   "));

    while (This != NULL) {

        //
        //  Copy characters including the semicolon, and add a newline and
        //  NULL.  We have three spaces at the beginning, so we need 6
        //  extra characters plus the size of the element itself.  It'd be
        //  nice if msvcrt didn't puke when the source string is larger
        //  than the specified size so we could just use a single
        //  stprintf_s for both cases.
        //

        for (i = 0; This[i] != '\0' && This[i] != ';'; i++) {
            Line[i + 3] = This[i];
        }
        Line[i + 3] = ';';
        Line[i + 4] = '\n';
        Line[i + 5] = '\0';

        if (!SdirWriteString(Line)) {
            return FALSE;
        }

        This += i;
        if (*This) {
            This++;
        } else {
            break;
        }
    }


    return TRUE;
}
#endif

#ifdef SDIR_SUPPORT_CUSTOM_METACOLOR

const
CHAR strMetaColorHeader[] = "METADATA COLOR CODING";

const
CHAR strMetaColorUsage1[] =
                   " Color coding for metadata attributes is defined via SDIR_COLOR_METADATA.\n"
                   " This variable also contains a semicolon delimited list of rules.  Each rule\n"
                   " takes the form of:\n"
                   "\n"
                   "   [file attribute],[color]\n"
                   "\n"
                   " Valid metadata attributes and their current defaults are:\n";

const
CHAR strMetaColorUsage2[] = "\n"
                   " For metadata colors, the keyword 'file' indicates to use the file color,\n"
                   " and not apply any specific metadata color.\n";


BOOL
SdirUsageMetaColor()
{
    TCHAR   Line[80];
    TCHAR   strUsage[1024];
    DWORD   i;

    if (!SdirUsageHeader(strMetaColorHeader)) {
        return FALSE;
    }

    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strMetaColorUsage1);
    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    for (i = 0; i < SdirGetNumSdirOptions(); i++) {
        if ((SdirOptions[i].Default.Flags & SDIR_FEATURE_FIXED_COLOR) == 0) {

            PSDIR_FEATURE Feature;

            Feature = SdirFeatureByOptionNumber(i);

            SdirColorStringFromFeature(Feature, strUsage, sizeof(strUsage)/sizeof(strUsage[0]));

            _stprintf_s(Line,
                        sizeof(Line)/sizeof(Line[0]),
                        _T("   %-25hs %s,%s;\n"),
                        SdirOptions[i].Help,
                        SdirOptions[i].Switch,
                        strUsage);

            if (!SdirWriteString(Line)) {
                return FALSE;
            }
        }
    }

    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strMetaColorUsage2);
    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    return TRUE;
}
#endif

#ifdef SDIR_SUPPORT_VT

const
CHAR strVtHeader[] = "VT OPTIONS";

const
CHAR strVtUsage1[] =
                   " VT escape sequences, often known as ANSI escape sequences, can be used to\n"
                   " provide text formatting based on the NTTERM environment variable.  This is\n"
                   " semicolon delimited and may contain one or more of the following options:\n"
                   "\n"
                   "   color;          indicates use of color\n"
                   "   extendedchars;  indicates the device will render extended ASCII characters\n"
                   "\n";

BOOL
SdirUsageVt()
{
    TCHAR   strUsage[1024];

    if (!SdirUsageHeader(strVtHeader)) {
        return FALSE;
    }

    SdirAnsiToUnicode(strUsage, sizeof(strUsage)/sizeof(strUsage[0]), strVtUsage1);
    if (!SdirWriteString(strUsage)) {
        return FALSE;
    }

    return TRUE;
}

#endif


BOOL
SdirUsage(
    __in int argc,
    __in_ecount(argc) LPTSTR argv[]
    )
{
    ULONG CurrentArg;
    BOOLEAN DisplayOptionsHelp = FALSE;
    BOOLEAN DisplayDisplayHelp = FALSE;
    BOOLEAN DisplaySortHelp = FALSE;
    BOOLEAN DisplayFileColorHelp = FALSE;
    BOOLEAN DisplayLicenseHelp = FALSE;
    BOOLEAN DisplayMetaColorHelp = FALSE;
    BOOLEAN DisplayOnlineHelp = FALSE;
    BOOLEAN DisplayVtHelp = FALSE;
    BOOLEAN DisplaySomething = FALSE;

    //
    //  Display current version.
    //

    if (!SdirUsageVersionInfo()) {
        return FALSE;
    }

    //
    //  Check if the user wants detailed information on something.
    //

    for (CurrentArg = 1; CurrentArg < (ULONG)argc; CurrentArg++) {

        if (argv[CurrentArg][0] == '-' || argv[CurrentArg][0] == '/') {
            LPTSTR Opt = &argv[CurrentArg][1];

            if (_tcsicmp(Opt, _T("opts")) == 0) {
                DisplayOptionsHelp = TRUE;
                DisplaySomething = TRUE;
            } else if (_tcsicmp(Opt, _T("display")) == 0) {
                DisplayDisplayHelp = TRUE;
                DisplaySomething = TRUE;
            } else if (_tcsicmp(Opt, _T("sort")) == 0) {
                DisplaySortHelp = TRUE;
                DisplaySomething = TRUE;
#ifdef SDIR_SUPPORT_CUSTOM_FILECOLOR
            } else if (_tcsicmp(Opt, _T("filecolor")) == 0) {
                DisplayFileColorHelp = TRUE;
                DisplaySomething = TRUE;
#endif
#ifdef SDIR_SUPPORT_CUSTOM_METACOLOR
            } else if (_tcsicmp(Opt, _T("metacolor")) == 0) {
                DisplayMetaColorHelp = TRUE;
                DisplaySomething = TRUE;
#endif
            } else if (_tcsicmp(Opt, _T("license")) == 0) {
                DisplayLicenseHelp = TRUE;
                DisplaySomething = TRUE;
#ifdef SDIR_SUPPORT_UPDATE
            } else if (_tcsicmp(Opt, _T("online")) == 0) {
                DisplayOnlineHelp = TRUE;
                DisplaySomething = TRUE;
#endif
            } else if (_tcsicmp(Opt, _T("v")) == 0) {
                DisplaySomething = TRUE;
#ifdef SDIR_SUPPORT_VT
            } else if (_tcsicmp(Opt, _T("vt")) == 0) {
                DisplayVtHelp = TRUE;
                DisplaySomething = TRUE;
#endif
            } else if (_tcsicmp(Opt, _T("all")) == 0) {
                DisplayOptionsHelp = TRUE;
                DisplayLicenseHelp = TRUE;
                DisplayDisplayHelp = TRUE;
                DisplaySortHelp = TRUE;
                DisplayFileColorHelp = TRUE;
                DisplayMetaColorHelp = TRUE;
                DisplayOnlineHelp = TRUE;
                DisplayVtHelp = TRUE;
                DisplaySomething = TRUE;
            }
        }
    }

    //
    //  If not, display help about what help topics they can select
    //  and return.
    //

    if (!DisplaySomething) {
        return SdirUsageHelp();
    }

    //
    //  Otherwise, display the detailed topics.
    //

    if (DisplayLicenseHelp && !SdirUsageLicense()) {
        return FALSE;
    }

    if (DisplayOptionsHelp && !SdirUsageOpts()) {
        return FALSE;
    }

    if (DisplayDisplayHelp && !SdirUsageDisplay()) {
        return FALSE;
    }

    if (DisplaySortHelp && !SdirUsageSort()) {
        return FALSE;
    }

#ifdef SDIR_SUPPORT_CUSTOM_FILECOLOR
    if (DisplayFileColorHelp && !SdirUsageFileColor()) {
        return FALSE;
    }
#endif

#ifdef SDIR_SUPPORT_CUSTOM_METACOLOR
    if (DisplayMetaColorHelp && !SdirUsageMetaColor()) {
        return FALSE;
    }
#endif

#ifdef SDIR_SUPPORT_VT
    if (DisplayVtHelp && !SdirUsageVt()) {
        return FALSE;
    }
#endif

#ifdef SDIR_SUPPORT_UPDATE
    if (DisplayOnlineHelp && !SdirUsageOnline()) {
        return FALSE;
    }
#endif

    return TRUE;
}

#else

BOOL
SdirUsage(
    __in int argc,
    __in_ecount(argc) LPTSTR argv[]
    )
{
    return TRUE;
}

#endif

// vim:sw=4:ts=4:et:
